{% load i18n %}
<h3>{% blocktrans with filter_title=title %} {{ filter_title }} {% endblocktrans %}</h3>
<ul>
  <input type="text" value="{{ spec.value|default_if_none:'' }}" name="{{ spec.parameter_name }}">
</ul>
